@@ -5,6 +5,7 @@ pass the names of specific files to format instead.
 """
 
 import functools
+import os
 import re
 import sys
 import textwrap
@@ -17,15 +18,14 @@ import black
 import isort
 import pyupgrade._main
 from black.mode import TargetVersion
-from black.parsing import lib2to3_parse
+from black.parsing import InvalidInput, lib2to3_parse
 from isort.exceptions import FileSkipComment
 
-__version__ = "0.9.6"
+__version__ = "0.10.3"
 __all__ = ["shed", "docshed"]
 
 # Conditionally imported in refactor mode to reduce startup latency in the common case
 com2ann: Any = None
-_teyit_refactor: Any = None
 _run_codemods: Any = None
 
 _version_map = {
@@ -71,15 +71,21 @@ def shed(
         return ""
 
     # Use black to autodetect our target versions
+    target_versions = {k for k, v in _version_map.items() if v >= min_version}
     try:
-        parsed = lib2to3_parse(
-            source_code.lstrip(),
-            target_versions={k for k, v in _version_map.items() if v >= min_version},
-        )
+        # Black errors out if we have pattern matching (3.10+), and the target versions
+        # include older versions.  Very well; we'll add a temporary workaround.
+        try:
+            parsed = lib2to3_parse(source_code.lstrip(), target_versions)
+        except InvalidInput as err:
+            if not re.match(r"Cannot parse: \d+:\d+: match ", str(err)):
+                raise
+            target_versions = {k for k, v in _version_map.items() if v >= (3, 10)}
+            parsed = lib2to3_parse(source_code.lstrip(), target_versions)
         # black.InvalidInput, blib2to3.pgen2.tokenize.TokenError, SyntaxError...
         # for forwards-compatibility I'm just going general here.
     except Exception as err:
-        msg = f"Could not parse {_location}"
+        msg = f"Could not parse {_location}\n    {type(err).__qualname__}: {err}"
         for pattern, blocktype in _SUGGESTIONS:
             if re.search(pattern, source_code, flags=re.MULTILINE):
                 msg += f"\n    Perhaps you should use a {blocktype!r} block instead?"
@@ -88,9 +94,14 @@ def shed(
         except SyntaxError:
             pass
         else:
-            msg += "\n    The syntax is valid Python, so please report this as a bug."
+            msg += (
+                f"\n    The syntax is valid for Python {sys.version_info.major}"
+                f".{sys.version_info.minor}, so please report this as a bug."
+            )
         w = ShedSyntaxWarning(msg)
         w.__cause__ = err
+        if "SHED_RAISE" in os.environ:  # pragma: no cover
+            raise w
         warnings.warn(w, stacklevel=_location.count(" block in ") + 2)
         # Even if the code itself has invalid syntax, we might be able to
         # regex-match and therefore reformat code embedded in docstrings.
@@ -101,7 +112,7 @@ def shed(
             min_version=min_version,
             _location=_location,
         )
-    target_versions = set(_version_map) & set(black.detect_target_versions(parsed))
+    target_versions &= set(black.detect_target_versions(parsed))
     assert target_versions
     min_version = max(
         min_version,
@@ -115,16 +126,10 @@ def shed(
         # This is a big deal for interactive use-cases such as pre-commit hooks
         # or format-on-save in editors (though I prefer Black for the latter).
         global com2ann
-        global _teyit_refactor
         global _run_codemods
         if com2ann is None:
             from ._codemods import _run_codemods  # type: ignore
 
-            try:
-                from teyit import refactor_until_deterministic as _teyit_refactor
-            except ImportError:  # pragma: no cover  # on Python 3.9
-                assert sys.version_info < (3, 9)
-                _teyit_refactor = _fallback
             try:
                 from com2ann import com2ann
             except ImportError:  # pragma: no cover  # on Python 3.8
@@ -145,8 +150,6 @@ def shed(
             # This can only be None if ast.parse() raises a SyntaxError,
             # which is possible but rare after the parsing checks above.
             source_code, _ = annotated
-        # Use teyit to replace old unittest.assertX methods on Python 3.9+
-        source_code, _ = _teyit_refactor(source_code)
 
     # One tricky thing: running `isort` or `autoflake` can "unlock" further fixes
     # for `black`, e.g. "pass;#" -> "pass\n#\n" -> "#\n".  We therefore run it
@@ -155,25 +158,17 @@ def shed(
     source_code = blackened = black.format_str(source_code, mode=black_mode)
 
     pyupgrade_min = min(min_version, max(pyupgrade._main.IMPORT_REMOVALS))
-    source_code = pyupgrade._main._fix_plugins(
-        source_code, settings=pyupgrade._main.Settings(min_version=pyupgrade_min)
-    )
+    pu_settings = pyupgrade._main.Settings(min_version=pyupgrade_min)
+    source_code = pyupgrade._main._fix_plugins(source_code, settings=pu_settings)
     if source_code != blackened:
-        # Second step to converge: https://github.com/asottile/pyupgrade/issues/273
-        source_code = pyupgrade._main._fix_plugins(
-            source_code, settings=pyupgrade._main.Settings(min_version=pyupgrade_min)
-        )
+        # Second step to converge: without this we have f-string problems.
+        # Upstream issue: https://github.com/asottile/pyupgrade/issues/703
+        source_code = pyupgrade._main._fix_plugins(source_code, settings=pu_settings)
     source_code = pyupgrade._main._fix_tokens(source_code, min_version=pyupgrade_min)
 
     if refactor:
         source_code = _run_codemods(source_code, min_version=min_version)
 
-    source_code = autoflake.fix_code(
-        source_code,
-        expand_star_imports=True,
-        remove_all_unused_imports=_remove_unused_imports,
-    )
-
     try:
         source_code = isort.code(
             source_code,
@@ -185,6 +180,12 @@ def shed(
     except FileSkipComment:
         pass
 
+    source_code = autoflake.fix_code(
+        source_code,
+        expand_star_imports=True,
+        remove_all_unused_imports=_remove_unused_imports,
+    )
+
     if source_code != blackened:
         source_code = black.format_str(source_code, mode=black_mode)
 
