@@ -5,13 +5,13 @@ These are mostly based on flake8, flake8-comprehensions, and some personal
 nitpicks about typing unions and literals.
 """
 
+import os
 import re
 from ast import literal_eval
 from typing import Tuple
 
 import libcst as cst
 import libcst.matchers as m
-from libcst._parser.types.config import _pick_compatible_python_version
 from libcst.codemod import VisitorBasedCodemodCommand
 
 
@@ -28,7 +28,7 @@ def attempt_hypothesis_codemods(context, mod):  # pragma: no cover
 
 
 imports_hypothesis = re.compile(
-    r"^ *(?:import hypothesis|from hypothesis(?:\.[a-z]+)* import )"
+    r"^ *(?:import hypothesis|from hypothesis(?:\.[a-z]+)* import )", flags=re.MULTILINE
 ).search
 
 
@@ -36,23 +36,16 @@ def _run_codemods(code: str, min_version: Tuple[int, int]) -> str:
     """Run all Shed fixers on a code string."""
     context = cst.codemod.CodemodContext()
 
-    # We want LibCST to parse the code as if running on our target minimum version,
-    # but fall back to the latest version it supports (currently 3.8) if our target
-    # version is newer than that.  Or jump *forward* if Black got the version wrong!
+    # Only the native parser supports Python 3.9 and later, but for now it's
+    # only active if you set an environment variable.  Very well then:
+    var = os.environ.get("LIBCST_PARSER_TYPE")
     try:
-        v = _pick_compatible_python_version(".".join(map(str, min_version)))
-        config = cst.PartialParserConfig(python_version=f"{v.major}.{v.minor}")
-        mod = cst.parse_module(code, config)
-    except cst.ParserSyntaxError as err:
-        # missed f-string version check https://github.com/Zac-HD/shed/issues/31
-        msg = "Incomplete input. Encountered '=', but expected '!', ':', or '}'."
-        if min_version >= (3, 8) or "=}" not in code or msg not in str(err):
-            raise  # pragma: no cover  # no *known* cases trigger this, but...
-        mod = cst.parse_module(code, cst.PartialParserConfig(python_version="3.8"))
-        # Right here, we know that the minimum version was too low.  We could in
-        # principle `return shed.shed(code, ...)` to fully account for that, but
-        # I don't want to pass around so many extra args just to cover an edge
-        # case which should be fixed soon regardless.
+        os.environ["LIBCST_PARSER_TYPE"] = "native"
+        mod = cst.parse_module(code)
+    finally:
+        os.environ.pop("LIBCST_PARSER_TYPE")
+        if var is not None:
+            os.environ["LIBCST_PARSER_TYPE"] = var
 
     if imports_hypothesis(code):  # pragma: no cover
         mod = attempt_hypothesis_codemods(context, mod)
@@ -73,6 +66,13 @@ def multi(*args, **kwargs):
     return m.OneOf(*(a(**kwargs) for a in args))
 
 
+def remove_trailing_comma(node):
+    # Remove the comma from this node, *unless* it's already a comma node with comments
+    if node.comma is cst.MaybeSentinel.DEFAULT or m.findall(node, m.Comment()):
+        return node
+    return node.with_changes(comma=cst.MaybeSentinel.DEFAULT)
+
+
 MATCH_NONE = m.MatchIfTrue(lambda x: x is None)
 ALL_ELEMS_SLICE = m.Slice(
     lower=MATCH_NONE | m.Name("None"),
@@ -114,21 +114,6 @@ class ShedFixers(VisitorBasedCodemodCommand):
             cst.Call(cst.Name("AssertionError"), args=[cst.Arg(updated_node.msg)])
         )
 
-    @m.leave(
-        m.Subscript(
-            value=m.Name(value="Optional"),
-            slice=[
-                m.SubscriptElement(
-                    slice=m.Index(value=m.Subscript(value=m.Name(value="Literal")))
-                )
-            ],
-        )
-    )
-    def convert_optional_literal_to_literal_none(self, _, updated_node):
-        expr = updated_node.slice[0].slice.value
-        none = cst.SubscriptElement(slice=cst.Index(value=cst.Name(value="None")))
-        return expr.with_changes(slice=list(expr.slice) + [none])
-
     @m.leave(
         m.ComparisonTarget(
             comparator=oneof_names("None", "False", "True"), operator=m.Equal()
@@ -227,9 +212,7 @@ class ShedFixers(VisitorBasedCodemodCommand):
                         args[i] = arg.with_changes(value=cst.Name("True"))
                     else:
                         del args[i]
-                        args[i - 1] = args[i - 1].with_changes(
-                            comma=cst.MaybeSentinel.DEFAULT
-                        )
+                        args[i - 1] = remove_trailing_comma(args[i - 1])
                 break
         else:
             args.append(cst.Arg(keyword=cst.Name("reverse"), value=cst.Name("True")))
@@ -294,14 +277,15 @@ class ShedFixers(VisitorBasedCodemodCommand):
             return cst.Call(func=func, args=[cst.Arg(updated_node.for_in.iter)])
         return updated_node
 
-    @m.leave(
-        m.Subscript(value=m.Name(value="Union"))
-        | m.Subscript(value=m.Name(value="Literal"))
-    )
+    @m.leave(m.Subscript(oneof_names("Union", "Literal")))
     def reorder_union_literal_contents_none_last(self, _, updated_node):
-        subscript_slice = list(updated_node.slice)
-        subscript_slice.sort(key=lambda elt: elt.slice.value.value == "None")
-        return updated_node.with_changes(slice=subscript_slice)
+        subscript = list(updated_node.slice)
+        try:
+            subscript.sort(key=lambda elt: elt.slice.value.value == "None")
+            subscript[-1] = remove_trailing_comma(subscript[-1])
+            return updated_node.with_changes(slice=subscript)
+        except Exception:  # Single-element literals are not slices, etc.
+            return updated_node
 
     @m.call_if_inside(m.Annotation(annotation=m.BinaryOperation()))
     @m.leave(
@@ -326,29 +310,18 @@ class ShedFixers(VisitorBasedCodemodCommand):
         else:
             return updated_node
 
-    @m.call_if_inside(m.Annotation(annotation=m.BinaryOperation()))
-    @m.leave(
-        m.BinaryOperation(
-            left=m.Subscript(value=m.Name(value="Literal")),
-            operator=m.BitOr(),
-            right=m.Name("None"),
-        )
-    )
-    def flatten_literal_op(self, _, updated_node):
-        literal = updated_node.left
-        args = list(literal.slice)
-        for item in args:
-            if m.matches(item, m.SubscriptElement(m.Index(m.Name("None")))):
-                return literal  # Already has "None"
-        args.append(
-            cst.SubscriptElement(
-                slice=cst.Index(value=cst.Name(value="None")),
-            )
-        )
-        return literal.with_changes(slice=tuple(args))
+    @m.leave(m.Subscript(value=m.Name("Literal")))
+    def flatten_literal_subscript(self, _, updated_node):
+        new_slice = []
+        for item in updated_node.slice:
+            if m.matches(item.slice.value, m.Subscript(m.Name("Literal"))):
+                new_slice += item.slice.value.slice
+            else:
+                new_slice.append(item)
+        return updated_node.with_changes(slice=new_slice)
 
-    @m.leave(m.Subscript(value=m.Name(value="Union") | m.Name(value="Literal")))
-    def flatten_union_literal_subscript(self, _, updated_node):
+    @m.leave(m.Subscript(value=m.Name("Union")))
+    def flatten_union_subscript(self, _, updated_node):
         new_slice = []
         has_none = False
         for item in updated_node.slice:
@@ -356,7 +329,7 @@ class ShedFixers(VisitorBasedCodemodCommand):
                 new_slice += item.slice.value.slice  # peel off "Optional"
                 has_none = True
             elif m.matches(
-                item.slice.value, m.Subscript(m.Name("Union") | m.Name("Literal"))
+                item.slice.value, m.Subscript(m.Name("Union"))
             ) and m.matches(updated_node.value, item.slice.value.value):
                 new_slice += item.slice.value.slice  # peel off "Union" or "Literal"
             elif m.matches(item.slice.value, m.Name("None")):
@@ -366,3 +339,51 @@ class ShedFixers(VisitorBasedCodemodCommand):
         if has_none:
             new_slice.append(cst.SubscriptElement(slice=cst.Index(cst.Name("None"))))
         return updated_node.with_changes(slice=new_slice)
+
+    @m.leave(m.Else(m.IndentedBlock([m.SimpleStatementLine([m.Pass()])])))
+    def discard_empty_else_blocks(self, _, updated_node):
+        # An `else: pass` block can always simply be discarded, and libcst ensures
+        # that an Else node can only ever occur attached to an If, While, For, or Try
+        # node; in each case `None` is the valid way to represent "no else block".
+        if m.findall(updated_node, m.Comment()):
+            return updated_node  # If there are any comments, keep the node
+        return cst.RemoveFromParent()
+
+    @m.leave(
+        m.Lambda(
+            params=m.MatchIfTrue(
+                lambda node: (
+                    node.star_kwarg is None
+                    and not node.kwonly_params
+                    and not node.posonly_params
+                    and isinstance(node.star_arg, cst.MaybeSentinel)
+                    and all(param.default is None for param in node.params)
+                )
+            )
+        )
+    )
+    def remove_lambda_indirection(self, _, updated_node):
+        same_args = [
+            m.Arg(m.Name(param.name.value), star="", keyword=None)
+            for param in updated_node.params.params
+        ]
+        if m.matches(updated_node.body, m.Call(args=same_args)):
+            return cst.ensure_type(updated_node.body, cst.Call).func
+        return updated_node
+
+    @m.leave(
+        m.BooleanOperation(
+            left=m.Call(m.Name("isinstance"), [m.Arg(), m.Arg()]),
+            operator=m.Or(),
+            right=m.Call(m.Name("isinstance"), [m.Arg(), m.Arg()]),
+        )
+    )
+    def collapse_isinstance_checks(self, _, updated_node):
+        left_target, left_type = updated_node.left.args
+        right_target, right_type = updated_node.right.args
+        if left_target.deep_equals(right_target):
+            merged_type = cst.Arg(
+                cst.Tuple([cst.Element(left_type.value), cst.Element(right_type.value)])
+            )
+            return updated_node.left.with_changes(args=[left_target, merged_type])
+        return updated_node
